import os
from dataclasses import dataclass

from pydantic import BaseModel, Field
from pydantic_ai import Agent, RunContext

# os.environ["GEMINI_API_KEY"] = "AIzaSyBqQsF_Tk7dp-S3iri0E__2DTzYmOZ0DD4"


class DatabaseConn:
    """This is a fake database for example purposes.

    In reality, you'd be connecting to an external database
    (e.g. PostgreSQL) to get information about customers.
    """

    @classmethod
    async def customer_name(cls, *, id: int) -> str | None:
        """Retrieve customer name from the database by their ID.

        Args:
            id (int): The customer's unique identifier

        Returns:
            str | None: The customer's name if found, None otherwise
        """
        if id == 123:
            return "John"

    @classmethod
    async def customer_balance(cls, *, id: int, include_pending: bool) -> float:
        """Retrieve customer's account balance from the database.

        Args:
            id (int): The customer's unique identifier
            include_pending (bool): Whether to include pending transactions in the balance

        Returns:
            float: The customer's current account balance

        Raises:
            ValueError: If the customer is not found
        """
        if id == 123:
            return 123.45
        else:
            raise ValueError("Customer not found")


@dataclass
class SupportDependencies:
    customer_id: int
    db: DatabaseConn


class SupportResult(BaseModel):
    support_advice: str = Field(description="Advice returned to the customer")
    block_card: bool = Field(description="Whether to block their")
    risk: int = Field(description="Risk level of query", ge=0, le=10)


support_agent = Agent(
    "gemini-1.5-flash",
    deps_type=SupportDependencies,
    result_type=SupportResult,
    system_prompt=(
        "You are a support agent in our bank, give the "
        "customer support and judge the risk level of their query. "
        "Reply using the customer's name."
    ),
)


@support_agent.system_prompt
async def add_customer_name(ctx: RunContext[SupportDependencies]) -> str:
    """Add customer name to the run context.

    Args:
        ctx (RunContext[SupportDependencies]): The run context containing customer dependencies
    """
    customer_name = await ctx.deps.db.customer_name(id=ctx.deps.customer_id)
    return f"The customer's name is {customer_name!r}"


@support_agent.tool
async def customer_balance(
    ctx: RunContext[SupportDependencies], include_pending: bool
) -> str:
    """Returns the customer's current account balance."""
    balance = await ctx.deps.db.customer_balance(
        id=ctx.deps.customer_id,
        include_pending=include_pending,
    )
    return f"${balance:.2f}"


if __name__ == "__main__":
    deps = SupportDependencies(customer_id=123, db=DatabaseConn())
    result = support_agent.run_sync("What is my balance?", deps=deps)
    print(result.data)

    result = support_agent.run_sync("I just lost my card!", deps=deps)
    print(result.data)
